using System;
using System.Collections;
using Server.Targeting;
using Server.Network;

// Ideas
// When you run on animals the panic
// When if ( distance < 8 && Utility.RandomDouble() * Math.Sqrt( (8 - distance) / 6 ) >= incoming.Skills[SkillName.AnimalTaming].Value )
// More your close, the more it can panic
/*
 * AnimalHunterAI, AnimalHidingAI, AnimalDomesticAI...
 * 
 */ 

namespace Server.Mobiles
{
	public class AnimalAI : BaseAI
	{
		private enum AnimalState
		{
			AnimalWander,	// Act normaly
			AnimalBackoff,	// Just go away
			AnimalFlee,		// Try to flee
			MeeleeAttack
		}

		AnimalState m_iState;

		public AnimalAI(BaseCreature m) : base (m)
		{
			m_iState = AnimalState.AnimalWander;
		}

		public override bool Think()
		{
			if ( m_Mobile.Deleted )
				return false;

			switch ( m_iState )
			{
				case AnimalState.AnimalWander:
					DoWander();
					break;

				case AnimalState.AnimalBackoff:
					DoBackOff();
					break;

				case AnimalState.AnimalFlee:
					DoFlee();
					break;

				case AnimalState.MeeleeAttack:
					break;
			}

			return true;
		}

		public virtual void DoWander()
		{
			if (AquireFocusMob(m_Mobile.RangePerception, AcquireType.Closest, true, false, true))
			{
				m_Mobile.DebugSay( "There is something near, I go away" );
				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = AnimalState.AnimalBackoff;
			}
			else
			{
				m_Mobile.DebugSay( "I am safe" );						
				DoWalkRandomInHome(2, 2, 1);
			}

			if ( m_Mobile.IsHurt() )
			{
				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = AnimalState.AnimalFlee;
			}
		}

		public virtual void DoBackOff()
		{
			if ( m_Mobile.IsHurt() )
			{
				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = AnimalState.AnimalFlee;
			}
			else
			{
				if (AquireFocusMob(m_Mobile.RangePerception * 3 / 2, AcquireType.Closest, true, false , true))
				{
					if ( DoWalkMobileRange(m_Mobile.FocusMob, 1, false, m_Mobile.RangePerception * 3 / 2, m_Mobile.RangePerception * 3) )
					{
						m_Mobile.DebugSay( "Well, here I am safe" );
						m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
						m_iState = AnimalState.AnimalWander;
					}					
				}
				else
				{
					m_Mobile.DebugSay( "I have lost my focus, let relax" );
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_iState = AnimalState.AnimalWander;
				}
			}
		}

		public virtual void DoFlee()
		{
			if (AquireFocusMob(m_Mobile.RangePerception * 3 / 2, AcquireType.Closest, true, false, true))
			{
				m_Mobile.DebugSay( "Something near, flee!" );
				DoWalkMobileRange(m_Mobile.FocusMob, 4, true, m_Mobile.RangePerception * 3 / 2, m_Mobile.RangePerception * 3);
			}

			if (!m_Mobile.IsHurt() )
			{
				m_Mobile.DebugSay( "I am not hurt anymore" );
				m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
				m_iState = AnimalState.AnimalWander;
			}
		}
	}
}