using System;
using System.Collections;
using Server.Targeting;
using Server.Network;
using Server.Spells;
using Server.Spells.First;
using Server.Spells.Second;
using Server.Spells.Fourth;

namespace Server.Mobiles
{
	public class HealerAI : BaseAI
	{
		public HealerAI( BaseCreature m ) : base( m )
		{
		}

		public override bool Think()
		{
			Target targ = m_Mobile.Target;

			if ( targ != null )
			{
				if ( targ is CureSpell.InternalTarget )
				{
					ProcessTarget( targ, new NeedDelegate( NeedCure ) );
				}
				else if ( targ is GreaterHealSpell.InternalTarget )
				{
					ProcessTarget( targ, new NeedDelegate( NeedGHeal ) );
				}
				else if ( targ is HealSpell.InternalTarget )
				{
					ProcessTarget( targ, new NeedDelegate( NeedLHeal ) );
				}
				else
				{
					targ.Cancel( m_Mobile, TargetCancelType.Canceled );
				}
			}
			else
			{
				Mobile toHelp = Find( new NeedDelegate( NeedCure ), new NeedDelegate( NeedGHeal ), new NeedDelegate( NeedLHeal ) );

				if ( toHelp != null )
				{
					if ( NeedCure( toHelp ) )
					{
						m_Mobile.DebugSay( "{0} needs a cure", toHelp.Name );

						if ( !(new CureSpell( m_Mobile, null )).Cast() )
							new CureSpell( m_Mobile, null ).Cast();
					}
					else if ( NeedGHeal( toHelp ) )
					{
						m_Mobile.DebugSay( "{0} needs a greater heal", toHelp.Name );

						if ( !(new GreaterHealSpell( m_Mobile, null )).Cast() )
							new HealSpell( m_Mobile, null ).Cast();
					}
					else if ( NeedLHeal( toHelp ) )
					{
						m_Mobile.DebugSay( "{0} needs a lesser heal", toHelp.Name );

						new HealSpell( m_Mobile, null ).Cast();
					}
				}
				else
				{
					if ( AquireFocusMob( m_Mobile.RangePerception, AcquireType.Weakest, false, true, false ) )
					{
						DoWalkMobileRange( m_Mobile.FocusMob, 1, false, 4, 7 );
					}
					else
					{
						DoWalkRandomInHome( 3, 2, 1 );
					}
				}
			}

			return true;
		}

		private delegate bool NeedDelegate( Mobile m );

		private void ProcessTarget( Target targ, NeedDelegate func )
		{
			Mobile toHelp = Find( func );

			if ( toHelp != null )
			{
				if ( targ.Range != -1 && !m_Mobile.InRange( toHelp, targ.Range ) )
				{
					m_Mobile.Move( m_Mobile.GetDirectionTo( toHelp ) | Direction.Running );
				}
				else
				{
					targ.Invoke( m_Mobile, toHelp );
				}
			}
			else
			{
				targ.Cancel( m_Mobile, TargetCancelType.Canceled );
			}
		}

		private Mobile Find( params NeedDelegate[] funcs )
		{
			if ( m_Mobile.Deleted )
				return null;

			Map map = m_Mobile.Map;

			if ( map != null )
			{
				IPooledEnumerable eable = map.GetMobilesInRange( m_Mobile.Location, m_Mobile.RangePerception );

				double prio = 0.0;
				Mobile found = null;

				foreach ( Mobile m in eable )
				{
					if ( !m_Mobile.CanSee( m ) || !(m is BaseCreature) || ((BaseCreature)m).Team != m_Mobile.Team )
						continue;

					for ( int i = 0; i < funcs.Length; ++i )
					{
						if ( funcs[i]( m ) )
						{
							double val = -m_Mobile.GetDistanceToSqrt( m );

							if ( found == null || val > prio )
							{
								prio = val;
								found = m;
							}

							break;
						}
					}
				}

				eable.Free();

				return found;
			}

			return null;
		}

		private bool NeedCure( Mobile m )
		{
			return m.Poisoned;
		}

		private bool NeedGHeal( Mobile m )
		{
			return m.Hits < m.HitsMax - 40;
		}

		private bool NeedLHeal( Mobile m )
		{
			return m.Hits < m.HitsMax - 10;
		}
	}
}