// Craft System Script v.0.2 by SickLab

using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public abstract class CraftSystem
	{
		private int m_MinCraftEffect;
		private int m_MaxCraftEffect;
		private double m_Delay;

		private CraftItemCol m_CraftItems;
		private CraftGroupCol m_CraftGroups;
		private CraftSubResCol m_CraftSubRes;

		public int MinCraftEffect { get { return m_MinCraftEffect; } }
		public int MaxCraftEffect { get { return m_MaxCraftEffect; } }
		public double Delay { get { return m_Delay; } }

		public CraftItemCol CraftItems{ get { return m_CraftItems; } }
		public CraftGroupCol CraftGroups{ get { return m_CraftGroups; } }
		public CraftSubResCol CraftSubRes{ get { return m_CraftSubRes; } }
		
		public abstract SkillName MainSkill{ get; }
		
		// For the Gump.  Title and X ident position
		public abstract string GumpTitle{ get; }
		public abstract int GumpTitlePos{ get; }

		public CraftSystem( int minCraftEffect, int maxCraftEffect, double delay )
		{
			m_MinCraftEffect = minCraftEffect;
			m_MaxCraftEffect = maxCraftEffect;
			m_Delay = delay;

			m_CraftItems = new CraftItemCol();
			m_CraftGroups = new CraftGroupCol();
			m_CraftSubRes = new CraftSubResCol();

			InitCraftList();
		}

		public void CreateItem( Mobile from, Type type, Type typeRes, BaseTool tool )
		{	
			// Verify if the type is in the list of the craftable item
			CraftItem craftItem = m_CraftItems.SearchFor( type );
			if ( craftItem != null )
			{
				// The item is in the list, try to create it
				craftItem.Craft( from, this, typeRes, tool );
			}
		}

		public int AddCraft( Type typeItem, string strGroup, string strName, double minSkill, double maxSkill, Type typeRes, string nameRes, int amount )
		{
			return AddCraft( typeItem, strGroup, strName, MainSkill, minSkill, maxSkill, typeRes, nameRes, amount, "" );
		}

		public int AddCraft( Type typeItem, string strGroup, string strName, double minSkill, double maxSkill, Type typeRes, string nameRes, int amount, int localizedMessage )
		{
			return AddCraft( typeItem, strGroup, strName, MainSkill, minSkill, maxSkill, typeRes, nameRes, amount, localizedMessage );
		}

		public int AddCraft( Type typeItem, string strGroup, string strName, double minSkill, double maxSkill, Type typeRes, string nameRes, int amount, string strMessage )
		{
			return AddCraft( typeItem, strGroup, strName, MainSkill, minSkill, maxSkill, typeRes, nameRes, amount, strMessage );
		}

		public int AddCraft( Type typeItem, string strGroup, string strName, SkillName skillToMake, double minSkill, double maxSkill, Type typeRes, string nameRes, int amount)
		{
			return AddCraft( typeItem, strGroup, strName, skillToMake, minSkill, maxSkill, typeRes, nameRes, amount, "" );
		}

		public int AddCraft( Type typeItem, string strGroup, string strName, SkillName skillToMake, double minSkill, double maxSkill, Type typeRes, string nameRes, int amount, int localizedMessage)
		{
			CraftItem craftItem = new CraftItem( typeItem, strGroup, strName );
			craftItem.AddRes( typeRes, nameRes, amount, localizedMessage );
			craftItem.AddSkill( skillToMake, minSkill, maxSkill );
			
			DoGroup( strGroup, craftItem );
			return m_CraftItems.Add( craftItem );
		}

		public int AddCraft( Type typeItem, string strGroup, string strName, SkillName skillToMake, double minSkill, double maxSkill, Type typeRes, string nameRes, int amount, string strMessage)
		{
			CraftItem craftItem = new CraftItem( typeItem, strGroup, strName );
			craftItem.AddRes( typeRes, nameRes, amount, strMessage );
			craftItem.AddSkill( skillToMake, minSkill, maxSkill );

			DoGroup( strGroup, craftItem );
			return m_CraftItems.Add( craftItem );
		}

		private void DoGroup( string strGroup, CraftItem craftItem )
		{
			
			int index = m_CraftGroups.SearchFor( strGroup );
			if ( index == -1)
			{
				CraftGroup craftGroup = new CraftGroup( strGroup );
				craftGroup.AddCraftItem( craftItem );
				m_CraftGroups.Add( craftGroup );
			}
			else
			{
				m_CraftGroups.GetAt( index ).AddCraftItem( craftItem );
			}
		}

		public void SetManaReq( int index, int mana )
		{
			CraftItem craftItem = m_CraftItems.GetAt( index );
			craftItem.Mana = mana;
		}

		public void SetStamReq( int index, int stam )
		{
			CraftItem craftItem = m_CraftItems.GetAt( index );
			craftItem.Stam = stam;
		}

		public void SetHitsReq( int index, int hits )
		{
			CraftItem craftItem = m_CraftItems.GetAt( index );
			craftItem.Hits = hits;
		}
		
		public void AddRes( int index, Type type, string name, int amount )
		{
			AddRes( index, type, name, amount, "" );
		}

		public void AddRes( int index, Type type, string name, int amount, int localizedMessage )
		{
			CraftItem craftItem = m_CraftItems.GetAt( index );
			craftItem.AddRes( type, name, amount, localizedMessage );
		}

		public void AddRes( int index, Type type, string name, int amount, string strMessage )
		{
			CraftItem craftItem = m_CraftItems.GetAt( index );
			craftItem.AddRes( type, name, amount, strMessage );
		}

		public void AddSkill( int index, SkillName skillToMake, double minSkill, double maxSkill )
		{
			CraftItem craftItem = m_CraftItems.GetAt(index);
			craftItem.AddSkill(skillToMake, minSkill, maxSkill);
		}

		public void SetSubRes( Type type, string name )
		{
			m_CraftSubRes.ResType = type;
			m_CraftSubRes.ResName = name;
			m_CraftSubRes.Init = true;
		}

		public void AddSubRes( Type type, string name, double level )
		{
			CraftSubRes craftSubRes = new CraftSubRes( type, name, level );
			m_CraftSubRes.Add( craftSubRes );
		}

		public abstract void InitCraftList();

		public abstract void PlayCraftEffect( Mobile from );
		public abstract void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality );

		public abstract bool CanCraft( Mobile from, BaseTool tool, Type itemType );
	}
}