// Tailoring Definition / Craft System Script v.0.2 by SickLab

using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public class DefTailoring : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Tailoring; }
		}

		public override string GumpTitle
		{
			get { return "TAILORING MENU"; }
		}

		public override int GumpTitlePos
		{
			get { return 185; }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefTailoring();

				return m_CraftSystem;
			}
		}

		private DefTailoring() : base( 1, 1, 4.5 )
		{
		}

		public override bool CanCraft( Mobile from, BaseTool tool, Type itemType )
		{
			if ( tool.Deleted )
			{
				return false;
			}
			else
			{
				if ( tool.Durability < 1 )
				{
					from.SendMessage( "This tool is broken" );
					return false;
				}
			}

			return true;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			from.PlaySound( 0x248 );
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality )
		{
			if ( toolBroken )
			{
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool
			}

			if ( failed )
			{
				if ( lostMaterial )
				{
					from.SendLocalizedMessage( 1044043 ); // You failed to create the item, and some of your materials are lost.
				}
				else
				{
					from.SendLocalizedMessage( 1044157 ); // You failed to create the item, but no materials were lost.
				}
			}
			else
			{
				if ( quality == 0 )
					from.SendLocalizedMessage( 502785 ); // You were barely able to make the item.  It's quality is below average
				else if (quality == 2 )
					from.SendLocalizedMessage( 502784 ); // Due to your exceptional skill, its quality is higher than average
				else				
					from.SendLocalizedMessage( 1044154 ); // You create the item
			}
		}

		public override void InitCraftList()
		{
			// Hats
			AddCraft( typeof( SkullCap ), "Hats", "Skullcap", 0.0, 20.0, typeof( Cloth ), "Cloth", 2);
			AddCraft( typeof( Bandana ), "Hats", "Bandana", 0.0, 20.0, typeof( Cloth ), "Cloth", 2);
			AddCraft( typeof( FloppyHat ), "Hats", "Floppy Hat", 6.2, 20.0, typeof( Cloth ), "Cloth", 11);
			AddCraft( typeof( Cap ), "Hats", "Cap", 6.2, 20.0, typeof( Cloth ), "Cloth", 11);
			AddCraft( typeof( WideBrimHat ), "Hats", "Wide-brim Hat", 6.2, 20.0, typeof( Cloth ), "Cloth", 12);
			AddCraft( typeof( StrawHat ), "Hats", "Straw Hat", 6.2, 20.0, typeof( Cloth ), "Cloth", 10);
			AddCraft( typeof( WizardsHat ), "Hats", "Wizard Hat", 7.2, 20.0, typeof( Cloth ), "Cloth", 15);
			AddCraft( typeof( Bonnet ), "Hats", "Bonnet", 6.2, 20.0, typeof( Cloth ), "Cloth", 11);
			AddCraft( typeof( FeatheredHat ), "Hats", "Feathered Hat", 6.2, 20.0, typeof( Cloth ), "Cloth", 12);
			AddCraft( typeof( TricorneHat ), "Hats", "Tricorne", 6.2, 20.0, typeof( Cloth ), "Cloth", 12);
			AddCraft( typeof( JesterHat ), "Hats", "Jester Hat", 7.2, 20.0, typeof( Cloth ), "Cloth", 15);
			AddCraft( typeof( TallStrawHat ), "Hats", "Tall Straw Hat", 6.7, 20.0, typeof( Cloth ), "Cloth", 13);

			// Shirts
			AddCraft( typeof( Doublet ), "Shirts", "Doublet", 10.0, 20.0, typeof( Cloth ), "Cloth", 8);
			AddCraft( typeof( Shirt ), "Shirts", "Shirt", 20.7, 40.0, typeof( Cloth ), "Cloth", 8);
			AddCraft( typeof( FancyShirt ), "Shirts", "Fancy Shirt", 24.8, 50.0, typeof( Cloth ), "Cloth", 8);
			AddCraft( typeof( Tunic ), "Shirts", "Tunic", 10.0, 20.0, typeof( Cloth ), "Cloth", 12);
			AddCraft( typeof( Surcoat ), "Shirts", "Surcoat", 8.2, 16.0, typeof( Cloth ), "Cloth", 14);
			AddCraft( typeof( PlainDress ), "Shirts", "Plain Dress", 12.4, 25.0, typeof( Cloth ), "Cloth", 10);
			AddCraft( typeof( FancyDress ), "Shirts", "Fancy Dress", 12.4, 25.0, typeof( Cloth ), "Cloth", 12);
			AddCraft( typeof( Robe ), "Shirts", "Robe", 53.9, 100.0, typeof( Cloth ), "Cloth", 16);
			AddCraft( typeof( Cloak ), "Shirts", "Cloak", 41.4, 80.0, typeof( Cloth ), "Cloth", 14);
			AddCraft( typeof( JesterSuit ), "Shirts", "Jester's Suit", 8.2, 20.0, typeof( Cloth ), "Cloth", 24);

			// Pants
			AddCraft( typeof( ShortPants ), "Pants", "Short Pants", 24.8, 50.0, typeof( Cloth ), "Cloth", 6);
			AddCraft( typeof( LongPants ), "Pants", "Long Pants", 24.8, 50.0, typeof( Cloth ), "Cloth", 8);
			AddCraft( typeof( Kilt ), "Pants", "Kilt", 20.7, 40.0, typeof( Cloth ), "Cloth", 8);
			AddCraft( typeof( Skirt ), "Pants", "Skirt", 29.0, 60.0, typeof( Cloth ), "Cloth", 10);

			// Misc
			AddCraft( typeof( BodySash ), "Miscellaneous", "BodySash", 4.1, 20.0, typeof( Cloth ), "Cloth", 4);
			AddCraft( typeof( HalfApron ), "Miscellaneous", "Half Apron", 20.7, 40.0, typeof( Cloth ), "Cloth", 6);
			AddCraft( typeof( FullApron ), "Miscellaneous", "Full Apron", 29.0, 60.0, typeof( Cloth ), "Cloth", 10);

			// Footwear
			AddCraft( typeof( Sandals ), "Footwear", "Sandals", 12.4, 25.0, typeof( Leather ), "Leather", 4);
			AddCraft( typeof( Shoes ), "Footwear", "Shoes", 16.5, 30.0, typeof( Leather ), "Leather", 6);
			AddCraft( typeof( Boots ), "Footwear", "Boots", 33.1, 65.0, typeof( Leather ), "Leather", 8);
			AddCraft( typeof( ThighBoots ), "Footwear", "Thigh Boots", 41.4, 80.0, typeof( Leather ), "Leather", 10);

			// Leather Armor
			AddCraft( typeof( LeatherGorget ), "Leather Armor", "Leather Gorget", 53.9, 90.0, typeof( Leather ), "Leather", 4);
			AddCraft( typeof( LeatherCap ), "Leather Armor", "Leather Cap", 6.2, 20.0, typeof( Leather ), "Leather", 2);
			AddCraft( typeof( LeatherGloves ), "Leather Armor", "Leather Gloves", 51.8, 90.0, typeof( Leather ), "Leather", 3);
			AddCraft( typeof( LeatherArms ), "Leather Armor", "Leather Sleeves", 53.9, 90.0, typeof( Leather ), "Leather", 4);
			AddCraft( typeof( LeatherLegs ), "Leather Armor", "Leather Leggings", 66.3, 100.0, typeof( Leather ), "Leather", 10);
			AddCraft( typeof( LeatherChest ), "Leather Armor", "Leather Tunic", 70.5, 100.0, typeof( Leather ), "Leather", 12);

			// Studded Armor
			AddCraft( typeof( StuddedGorget ), "Studded Armor", "Studded Gorget", 78.8, 100.0, typeof( Leather ), "Leather", 6);
			AddCraft( typeof( StuddedGloves ), "Studded Armor", "Studded Gloves", 82.9, 100.0, typeof( Leather ), "Leather", 8);
			AddCraft( typeof( StuddedArms ), "Studded Armor", "Studded Sleeves", 87.1, 100.0, typeof( Leather ), "Leather", 10);
			AddCraft( typeof( StuddedLegs ), "Studded Armor", "Studded Leggings", 91.2, 100.0, typeof( Leather ), "Leather", 12);
			AddCraft( typeof( StuddedChest ), "Studded Armor", "Studded Tunic", 95.4, 100.0, typeof( Leather ), "Leather", 14);

			// Female Armor
			AddCraft( typeof( LeatherShorts ), "Female Armor", "Leather Shorts", 62.2, 100.0, typeof( Leather ), "Leather", 8);
			AddCraft( typeof( LeatherSkirt ), "Female Armor", "Leather Skirt", 58.0, 100.0, typeof( Leather ), "Leather", 6);
			AddCraft( typeof( LeatherBustierArms ), "Female Armor", "Leather Bustier", 58.0, 100.0, typeof( Leather ), "Leather", 6);
			AddCraft( typeof( FemaleLeatherChest ), "Female Armor", "Leather Armor", 62.2, 100.0, typeof( Leather ), "Leather", 8);
			AddCraft( typeof( StuddedBustierArms ), "Female Armor", "Studded Bustier", 82.9, 100.0, typeof( Leather ), "Leather", 8);
			AddCraft( typeof( FemaleStuddedChest ), "Female Armor", "Studded Armor", 87.1, 100.0, typeof( Leather ), "Leather", 10);

			int index = -1;
			// Bone Armor
			index = AddCraft( typeof( BoneHelm ), "Bone Armor", "Bone Helmet", 85.0, 100.0, typeof( Leather ), "Leather", 4);
			AddRes( index, typeof( Bone ), "Bone", 2 );
			
			index = AddCraft( typeof( BoneGloves ), "Bone Armor", "Bone Glove", 89.0, 100.0, typeof( Leather ), "Leather", 6);
			AddRes( index, typeof( Bone ), "Bone", 2 );

			index = AddCraft( typeof( BoneArms ), "Bone Armor", "Bone Arms", 92.0, 100.0, typeof( Leather ), "Leather", 8);
			AddRes( index, typeof( Bone ), "Bone", 4 );

			index = AddCraft( typeof( BoneLegs ), "Bone Armor", "Bone Leggings", 95.0, 100.0, typeof( Leather ), "Leather", 10);
			AddRes( index, typeof( Bone ), "Bone", 6 );
		
			index = AddCraft( typeof( BoneChest ), "Bone Armor", "Bone Armor", 96.0, 100.0, typeof( Leather ), "Leather", 12);
			AddRes( index, typeof( Bone ), "Bone", 10 );

			// Set the overidable material
			SetSubRes( typeof( Leather ), "Leather" );

			// Add every material you want the player to be able to chose from
			// This will overide the overidable material
			AddSubRes( typeof( Leather ), "Leather", 1.0 );
			AddSubRes( typeof( SpinedLeather ), "Spined Leather", 1.1 );
			AddSubRes( typeof( HornedLeather ), "Horned Leather", 1.2 );
			AddSubRes( typeof( BarbedLeather ), "Barbed Leather", 1.3 );
		}
	}
}