// Tinkering Definition / Craft System Script v.0.2 by SickLab

using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public class DefTinkering : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Tinkering; }
		}

		public override string GumpTitle
		{
			get { return "TINKERING MENU"; }
		}

		public override int GumpTitlePos
		{
			get { return 185; }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefTinkering();

				return m_CraftSystem;
			}
		}

		private DefTinkering() : base( 1, 2, 3.0 )
		{
		}

		public override bool CanCraft( Mobile from, BaseTool tool, Type itemType )
		{
			if ( tool.Deleted )
			{
				return false;
			}
			else
			{
				if ( tool.Durability < 1 )
				{
					from.SendMessage( "This tool is broken" );
					return false;
				}
			}

			return true;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			from.PlaySound( 0x241 );
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality )
		{
			if ( toolBroken )
			{
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool
			}

			if ( failed )
			{
				if ( lostMaterial) 
				{
					from.SendLocalizedMessage( 1044043 ); // You failed to create the item, and some of your materials are lost.
				}
				else
				{
					from.SendLocalizedMessage( 1044157 ); // You failed to create the item, but no materials were lost.
				}
			}
			else
			{
				if ( quality == 0 )
					from.SendLocalizedMessage( 502785 ); // You were barely able to make the item.  It's quality is below average
				else if (quality == 2 )
					from.SendLocalizedMessage( 502784 ); // Due to your exceptional skill, its quality is higher than average
				else				
					from.SendLocalizedMessage( 1044154 ); // You create the item
			}
		}

		public override void InitCraftList()
		{
			// Tools
			AddCraft( typeof( Scissors ), "Tools", "Scissors", 5.0, 20.0, typeof( IronIngot ), "Iron Ingot", 2, 1044037 );
			AddCraft( typeof( MortarPestle ), "Tools", "Mortar and Pestle", 20.0, 40.0, typeof( IronIngot ), "Iron Ingot", 3, 1044037 );
			AddCraft( typeof( TinkerTools ), "Tools", "Tinker's tools", 10.0, 20.0, typeof( IronIngot ), "Iron Ingot", 2, 1044037 );
			AddCraft( typeof( Hatchet ), "Tools", "Hatchet", 30.0, 60.0, typeof( IronIngot ), "Iron Ingot", 4, 1044037 );
			AddCraft( typeof( SewingKit ), "Tools", "Sewing Kit", 10.0, 20.0, typeof( IronIngot ), "Iron Ingot", 2, 1044037 );
			AddCraft( typeof( Saw ), "Tools", "Saw", 30.0, 60.0, typeof( IronIngot ), "Iron Ingot", 4, 1044037 );
			AddCraft( typeof( SmithHammer ), "Tools", "Smith's Hammer", 40.0, 80.0, typeof( IronIngot ), "Iron Ingot", 4, 1044037 );
			AddCraft( typeof( Pickaxe ), "Tools", "Pickaxe", 40.0, 80.0, typeof( IronIngot ), "Iron Ingot", 4, 1044037 );
			AddCraft( typeof( Lockpick ), "Tools", "Lockpick", 45.0, 90.0, typeof( IronIngot ), "Iron Ingot", 1, 1044037 );
			AddCraft( typeof( ScribesPen ), "Tools", "Scribe's Pen", 45.0, 90.0, typeof( IronIngot ), "Iron Ingot", 1, 1044037 ); // What are the resources for this?

			// Utensils
			AddCraft( typeof( ButcherKnife ), "Utensils", "Butcher Knife", 25.0, 50.0, typeof( IronIngot ), "Iron Ingot", 2, 1044037 );
			AddCraft( typeof( Cleaver ), "Utensils", "Cleaver", 20.0, 40.0, typeof( IronIngot ), "Iron Ingot", 3, 1044037 );
			AddCraft( typeof( SkinningKnife ), "Utensils", "Skinning Knife", 25.0, 50.0, typeof( IronIngot ), "Iron Ingot", 2, 1044037 );

		}
	}
}