using System;
using Server;
using Server.Multis;
using Server.Network;

namespace Server.Items
{
	[FlipableAttribute( 0xe80, 0x9a8 )]
	public class StrongBox : Container
	{
		private Mobile m_Owner;
		private BaseHouse m_House;
		
		public StrongBox( Mobile owner, BaseHouse house ) : base( 0xE80, 0x4B, 0x42, new Rectangle2D( 16, 51, 168, 73 ) )
		{
			m_Owner = owner;
			m_House = house;
		}

		public Mobile Owner
		{
			get
			{
				return m_Owner;
			}
			set
			{
				m_Owner = value;
			}
		}

		public StrongBox( Serial serial ) : base(serial)
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			writer.Write( (int)0 );//version

			writer.Write( m_Owner );
			writer.Write( m_House );
		}

		public override void Deserialize( GenericReader reader )
		{
			int version = reader.ReadInt();

			m_Owner = reader.ReadMobile();
			m_House = reader.ReadItem() as BaseHouse;
		}

		public override bool Decays
		{
			get
			{
				return false;
			}
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( m_Owner != null && m_Owner.Deleted )
			{
				this.Delete();
				return;
			}

			if ( m_Owner == null || m_Owner.Serial == from.Serial || m_House.Owner.Serial == from.Serial )
			{
				DisplayTo( from );
			}
			else
			{
				from.SendMessage( "You do not own that strong box!" );
			}
		}

		public override void OnSingleClick( Mobile from )
		{
			if ( m_Owner != null && m_Owner.Deleted )
			{
				this.Delete();
				return;
			}

			if ( m_Owner != null )
			{
				LabelTo( from, 1042887, m_Owner.Name );//"a strong box owned by ~1_OWNER_NAME~"
				from.Send( new UnicodeMessage( Serial, ItemID, MessageType.Label, 0x3B2, 3, "ENU", "", String.Format( "({0} items, {1} stones)", TotalItems, TotalWeight ) ) );
			}
			else
				base.OnSingleClick( from );
		}

		public override bool OnDragDrop( Mobile from, Item dropped )
		{
			if ( m_Owner == null || m_Owner.Serial == from.Serial || m_House.Owner.Serial == from.Serial )
			{
				if ( MyTryDropItem( from, dropped, true ) )
				{
					from.Send( new PlaySound( GetDroppedSound( dropped ), from ) );

					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		public bool MyTryDropItem( Mobile from, Item dropped, bool sendFullMessage )
		{
			if ( (TotalItems + dropped.TotalItems + 1) > 25 )
			{
				if ( sendFullMessage )
					from.SendMessage( "That container cannot hold more items." );

				return false;
			}
			else
				return TryDropItem( from, dropped, sendFullMessage );
		}
	}
}
