using System;
using Server.Misc;

namespace Server.Items
{
	[DispellableFieldAttribute]
	public class Moongate : Item
	{
		private Point3D m_Target;
		private Map m_TargetMap;
		private bool m_bDispellable;

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D Target
		{
			get
			{
				return m_Target;
			}
			set
			{
				m_Target = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Map TargetMap
		{
			get
			{
				return m_TargetMap;
			}
			set
			{
				m_TargetMap = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Dispellable
		{
			get
			{
				return m_bDispellable;
			}
			set
			{
				m_bDispellable = value;
			}
		}


		[Constructable]
		public Moongate() : this( Point3D.Zero, null )
		{
			m_bDispellable = true;
		}

		public Moongate(bool bDispellable) : this( Point3D.Zero, null )
		{
			m_bDispellable = bDispellable;
		}

		public Moongate( Point3D target, Map targetMap ) : base( 0xF6C )
		{
			Movable = false;

			m_Target = target;
			m_TargetMap = targetMap;
		}

		public Moongate( Serial serial ) : base( serial )
		{
		}

		public override bool OnMoveOver( Mobile m )
		{
			if ( !m.Player )
				return true;

			if ( m_TargetMap != null )
			{
				m.Location = m_Target;
				m.Map = m_TargetMap;

				m.PlaySound( 0x1FE );

				return false;
			}
			else
			{
				m.SendMessage( "This moongate does not seem to go anywhere." );

				return true;
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( m_Target );
			writer.Write( m_TargetMap );
			
			// Version 1
			writer.Write( m_bDispellable );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			m_Target = reader.ReadPoint3D();
			m_TargetMap = reader.ReadMap();

			if (version >= 1)
			{
				m_bDispellable = reader.ReadBool();
			}
		}
	}
}