using Server;
using System;
using Server.Items;
using Server.Multis;
using Server.Spells;
using Server.Spells.Sixth;

namespace Server.Regions
{
	public class HouseRegion : Region
	{
		private BaseHouse m_House;

		public HouseRegion( BaseHouse house ) : base( "", "", house.Map )
		{
			Priority = Region.HousePriority;
			LoadFromFile = false;
			m_House = house;
		}

		public override void OnEnter( Mobile m )
		{
			if ( m_House.IsBanned( m ) )
			{
				m.Location = this.GoLocation;
				m.SendLocalizedMessage( 501340 );//You have been banned from this house
				return;
			}

			if ( m_House.IsFriend( m ) )
				m.SendLocalizedMessage( 501293 );//Welcome back to the house, friend.
		}

		public override bool OnDecay( Item item )
		{
			if ( m_House.IsLockedDown( item ) || m_House.IsSecure( item ) )
				return false;
			else
				return base.OnDecay(item);
		}

		public override TimeSpan GetLogoutDelay( Mobile m )
		{
			if ( m_House.IsFriend( m ) )
				return TimeSpan.FromSeconds( 0.5 );
			else
				return base.GetLogoutDelay( m );
		}

		public override void OnSpeech( SpeechEventArgs e )
		{
			bool isFriend, isCoOwn;
			isCoOwn = m_House.IsCoOwner( e.Mobile );

			if ( isCoOwn )
				isFriend = true;
			else
				isFriend = m_House.IsFriend( e.Mobile );

			if ( !isFriend && !isCoOwn )
				return;

			if ( isFriend )
			{
				if ( e.HasKeyword( 0x33 ) )//remove thyself
				{
					e.Mobile.Target = new HouseKickTarget( m_House );
				}

				if ( !isCoOwn )
					return;
			}

			if ( e.HasKeyword( 0x23 ) ) //I wish to lock this down
			{
				e.Mobile.Target = new LockdownTarget( false, m_House );
			}
			else if ( e.HasKeyword( 0x24 ) ) //I wish to release this
			{
				e.Mobile.Target = new LockdownTarget( true, m_House );
			}
			else if ( e.HasKeyword( 0x25 ) )//I wish to secure this
			{
				e.Mobile.Target = new SecureTarget( false, m_House );
			}
			else if ( e.HasKeyword( 0x26 ) )//I wish to unsecure this
			{
				e.Mobile.Target = new SecureTarget( true, m_House );
			}
			else if ( e.HasKeyword( 0x27 ) )//I wish to place a strong box
			{
				m_House.AddStrongBox( e.Mobile );
			}
			else if ( e.HasKeyword( 0x28 ) )//I wish to place a trash barrel
			{
				m_House.AddTrashBarrel( e.Mobile.Location );
			}
			else if ( e.HasKeyword( 0x34 ) ) //I ban thee
			{
				e.Mobile.Target = new HouseBanTarget( true, m_House );
			}
		}

		public override void OnExit( Mobile m )
		{
			//m.SendMessage( "Bye!" );
		}

		public override bool OnBeginSpellCast( Mobile m, Spell s )
		{
			if ( s is MarkSpell )
			{
				m.SendLocalizedMessage( 501800 );//You cannot mark that object at that location
				return false;
			}
			else
				return base.OnBeginSpellCast( m, s );
		}

		public override bool OnDoubleClick( Mobile from, object o )
		{
			if ( o is Container )
			{
				Container i = (Container)o;
				if ( m_House.IsSecure( i ) )
				{
					if ( m_House.IsCoOwner( from ) )
					{
						return true;
					}
					else
					{
						from.SendLocalizedMessage( 501655 );//That is secure!
						return false;
					}
				}
			}
			return true;
		}

		public override bool OnSingleClick( Mobile from, object o )
		{
			if ( o is Item )
			{
				Item i = (Item)o;
				if ( m_House.IsLockedDown( i ) )
				{
					i.LabelTo( from, 501643 );//[locked down]
				} else if ( m_House.IsSecure( i ) )
				{
					i.LabelTo( from, 501644 );//[locked down & secure]
				}
			}
			return true;
		}

		public BaseHouse House
		{
			get
			{
				return m_House;
			}
		}
	}
}