using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Fourth
{
	public class GreaterHealSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Greater Heal", "In Vas Mani",
				SpellCircle.Fourth,
				204,
				Reagent.Garlic,
				Reagent.Ginseng,
				Reagent.MandrakeRoot,
				Reagent.SpidersSilk
			);

		public GreaterHealSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( m.Poisoned )
			{
				Caster.SendLocalizedMessage( (Caster == m) ? 1005000 : 1010398 ); // Cannot be healed in current state.
			}
			else if ( CheckBSequence( m ) )
			{
				// Algorithm: (40% of magery) + (1-10)

				int toHeal = (int)(Caster.Skills[SkillName.Magery].Value * 0.4);
				toHeal += Utility.Random( 1, 10 );

				m.Heal( toHeal );

				if ( Caster != m )
				{
					Caster.Direction = Caster.GetDirectionTo( m );
				}

				m.PlaySound( 0x202 );
				Effects.SendTargetEffect( m, 0x376A, 16 );
			}

			FinishSequence();
		}

		public class InternalTarget : Target
		{
			private GreaterHealSpell m_Owner;

			public InternalTarget( GreaterHealSpell owner ) : base( 12, false, TargetFlags.Beneficial )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
				{
					m_Owner.Target( (Mobile)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}