// Mana Drain Spell Script v.1.0 by SickLab

using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Fourth
{
	public class ManaDrainSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Mana Drain", "Ort Rel",
				SpellCircle.Fourth,
				Reagent.BlackPearl,
				Reagent.MandrakeRoot,
				Reagent.SpidersSilk
			);

		public ManaDrainSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckHSequence( m ) )
			{
				SpellHelper.CheckReflect( (int)this.Circle, Caster, ref m );

				if ( CheckResisted( m ) )
					m.SendLocalizedMessage( 501783 ); // You feel yourself resisting magical energy.
				else
					m.Mana -= Utility.Random( 1, m.ManaMax );

				if ( Caster != m )
					Caster.Direction = Caster.GetDirectionTo( m );

				m.PlaySound( 0x1F8 );
				Effects.SendTargetEffect( m, 0x374A, 16 );
			}

			FinishSequence();
		}

		public override double GetResistPercent( Mobile target )
		{
			return base.GetResistPercentForCircle( target, SpellCircle.First );
		}

		private class InternalTarget : Target
		{
			private ManaDrainSpell m_Owner;

			public InternalTarget( ManaDrainSpell owner ) : base( 12, false, TargetFlags.Harmful )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
					m_Owner.Target( (Mobile)o );
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}