using System;
using Server;
using Server.Regions;

namespace Server.Spells
{
	public class SpellHelper
	{
		//stat mods
		public static void AddStatBonus( Mobile caster, Mobile target, StatType type )
		{
			int offset = GetOffset( caster );
			string name = String.Format( "[Spell] {0} Offset", type );

			StatMod mod = target.GetStatMod( name );

			if ( mod != null && mod.Offset < 0 )
			{
				target.AddStatMod( new StatMod( type, name, mod.Offset + offset, GetDuration( caster ) ) );
			}
			else if ( mod == null || mod.Offset < offset )
			{
				target.AddStatMod( new StatMod( type, name, offset, GetDuration( caster ) ) );
			}
		}

		public static void AddStatCurse( Mobile caster, Mobile target, StatType type )
		{
			int offset = -GetOffset( caster );
			string name = String.Format( "[Spell] {0} Offset", type );

			StatMod mod = target.GetStatMod( name );

			if ( mod != null && mod.Offset > 0 )
			{
				target.AddStatMod( new StatMod( type, name, mod.Offset + offset, GetDuration( caster ) ) );
			}
			else if ( mod == null || mod.Offset > offset )
			{
				target.AddStatMod( new StatMod( type, name, offset, GetDuration( caster ) ) );
			}
		}

		private static int GetOffset( Mobile caster )
		{
			return 1 + (int)(caster.Skills[SkillName.Magery].Value * 0.1);
		}

		private static TimeSpan GetDuration( Mobile caster )
		{
			return TimeSpan.FromSeconds( caster.Skills[SkillName.Magery].Value * 1.2 );
		}

		//towns
		public static bool IsTown( IPoint3D loc, Mobile caster )
		{
			if ( loc is Item )
				loc = ((Item)loc).GetWorldLocation();

			return IsTown( new Point3D( loc ), caster );
		}

		public static bool IsTown( Point3D loc, Mobile caster )
		{
			Map map = caster.Map;

			if ( map == null )
				return false;

			Region r = Region.Find( loc, map );

			return ( r is TrammelTown || r is FeluccaTown );
		}

		public static bool CheckTown( IPoint3D loc, Mobile caster )
		{
			if ( loc is Item )
				loc = ((Item)loc).GetWorldLocation();

			return CheckTown( new Point3D( loc ), caster );
		}

		public static bool CheckTown( Point3D loc, Mobile caster )
		{
			if ( IsTown( loc, caster ) )
			{
				caster.SendLocalizedMessage( 500946 ); // You cannot cast this in town!
				return false;
			}

			return true;
		}

		//magic reflection
		public static void CheckReflect( int Circle, Mobile caster, ref Mobile target )
		{
			CheckReflect( Circle, ref caster, ref target );
		}

		public static void CheckReflect( int Circle, ref Mobile caster, ref Mobile target )
		{
			if ( target.MagicDamageAbsorb > 0 )
			{
				if ( target.MagicDamageAbsorb > Circle )
				{
					target.MagicDamageAbsorb -= Circle;
					Mobile temp = caster;
					caster = target;
					target = temp;
				}
				else
				{
					target.MagicDamageAbsorb = 0;
					DefensiveSpell.Nullify( target );
				}
			}
		}
	}
}
